#!/usr/bin/env python

#
#
# ImageWorkshop
# Copyright 2002 Roland Stigge
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#

# Tkinter
import Tkinter
import tkMessageBox
import tkFileDialog

# own modules
import GUI              # common dialogs, elements
import ImageGUI         # Image handling
import ActionGUI        # Essential Workshop classes
import ImageWorkshop

version = '0.0.3' # XXX: Distutils support

# Some config, to be stored in config file and/or external

# Maximum initial image window size
#IMAGEWINDOWWIDTH = 256
#IMAGEWINDOWHEIGHT = 256

class Workshop(Tkinter.Tk):
    "The main window"
    def __init__(self):
        Tkinter.Tk.__init__(self)
        self.protocol("WM_DELETE_WINDOW", self.exit)

        self.title("ImageWorkshop")
        # create the main menu
        menu = Tkinter.Menu(self)

        self.config(menu = menu)
        self.geometry("%dx%d" % (400, 150))

        # status bar
        self.status = Tkinter.StringVar()
        self.position = Tkinter.StringVar()
        self.position.set("   ")
        outerstatus = Tkinter.Frame(self, relief = Tkinter.RAISED, bd = 1)
        outerstatus.pack(side = Tkinter.BOTTOM, fill = Tkinter.X)
        Tkinter.Label(outerstatus, textvariable = self.status, bd = 1,
                      relief = Tkinter.SUNKEN,
                      anchor = Tkinter.W).pack(padx = 2, pady = 2,
                                               fill = Tkinter.X,
                                               side = Tkinter.LEFT,
                                               expand = 1)
        Tkinter.Label(outerstatus, textvariable = self.position, bd = 1,
                      relief = Tkinter.SUNKEN,
                      anchor = Tkinter.W).pack(padx = 2, pady = 2,
                                               side = Tkinter.RIGHT)

        # start report
        self.text = Tkinter.Text(self, height = 128, bd = 0,
                                 state = Tkinter.DISABLED, wrap = Tkinter.NONE)
        self.text.pack(side = Tkinter.BOTTOM, anchor = Tkinter.S)
        self.report("Welcome to ImageWorkshop!")
        self.report("(2002 by Roland Stigge)\n")
        self.bind("<Configure>", self.seereport)
        
        menustructure = [
            ("cascade", "_File", "filename",
             [("command", "_Open...", self.open, "<Alt-o>"),
              ("cascade", "_Generate", "generatormenu",
               [("command", "_Homogeneous", self.generatehomogeneous, ""),
                ("command", "_Chess", self.generatechess, ""),
                ("command", "Gradien_t", self.generategradient, ""),
                ("command", "_Gauss", self.generategauss, ""),
                ("command", "_Dirac", self.generatedirac, ""),
                ("command", "_Sin / Cos", self.generatesin, ""),
                ("command", "S_iemens", self.generatesiemens, "")
                ]),
              ("separator",),
              ("command", "E_xit", self.exit, "<Alt-x>")
              ]),
            ("cascade", "_Help", "helpmenu",
             [("command", "_About...", self.about, "")
              ])
            ]

        GUI.fillmenu(menu, menustructure, self)

        self.windowregisterlist = [] #list where ImageWindows may be registered
    
        #XXX: contextsensitive help

        #debug: XXX
        #ActionGUI.FileImageWindow(self, "/home/ernie/code/ImageWorkshop/lena.gif")

    def report(self, line):
        "generates a new report line in main window"
        self.text.config(state = Tkinter.NORMAL)
        self.text.insert(Tkinter.END, "\n" + line)
        while int(self.text.index(Tkinter.END).split(".")[0]) > 256:
            while self.text.get("0.0") != "\n":
                self.text.delete("0.0")
            self.text.delete("0.0")
        self.text.config(state = Tkinter.DISABLED)
        self.seereport()

    def seereport(self, event = None):
        "sets the view to the end of the report"
        self.text.see(str(int(self.text.index(Tkinter.END).split(".")[0])
                          - 1) + ".0")

    def registerwindow(self, window):
        "registers an image window (an ImageWindow instance)"
        if window not in self.windowregisterlist:
            self.windowregisterlist.append(window)
            self.report("New image window: " + window.name)

    def unregisterwindow(self, window):
        "removes the specified window from list (undo self.registerwindow())"
        while window in self.windowregisterlist:
            self.windowregisterlist.remove(window)
            self.report("Closing image window: " + window.name)

    def registeredwindows(self):
        "returns list of registered windows"
        return self.windowregisterlist

    def open(self, event = None):
        "Open File was chosen from the menu"
        filename = tkFileDialog.askopenfilename(filetypes = # read types
                    [('All files','*'),
                     ('Microsoft Windows BMP','*.bmp'),
                     ('Graphics Interchange Format (GIF)','*.gif'),
                     ('Joint Photographic Expert Group (JPEG)','*.jpg'),
                     ('Joint Photographic Expert Group (JPEG)','*.jpe'),
                     ('Joint Photographic Expert Group (JPEG)','*.jpeg'),
                     ('PC Paintbrush (PCX)','*.pcx'),
                     ('Portable Network Graphics (PNG)','*.png'),
                     ('Portable Bitmap','*.pbm'),
                     ('Portable Bitmap','*.ppm'),
                     ('Portable Bitmap','*.pgm'),
                     ('Targa File (TGA)','*.tga'),
                     ('Tag Image File Format (TIFF)','*.tiff'),
                     ('Tag Image File Format (TIFF)','*.tif')])
        if filename: # valid filename chosen
            # create new image window
            ActionGUI.FileImageWindow(self, filename)
            
    def generatechess(self, event = None):
        d = ActionGUI.ChessDialog(self, title = "Chess")
        if d.width: # not canceled
            ActionGUI.ChessImageWindow(self, d.width, d.height, d.frequencyx,
                                       d.frequencyy, name = "Chess")

    def generategradient(self, event = None):
        d = ActionGUI.GradientDialog(self, title = "Gradient")
        if d.width: # not canceled
            ActionGUI.GradientImageWindow(self, d.width, d.height,
                                          d.modevar.get(), name = "Gradient")

    def generategauss(self, event = None):
        d = ActionGUI.GaussDialog(self, title = "Gauss")
        if d.width: # not canceled
            ActionGUI.GaussImageWindow(self, d.width, d.height, d.cbvarx.get(),
                                       d.stddevx, d.cbvary.get(), d.stddevy,
                                       name = "Gauss")
        
    def generatedirac(self, event = None):
        d = ActionGUI.DiracDialog(self, title = "Dirac")
        if d.width: # not canceled
            ActionGUI.DiracImageWindow(self, d.width, d.height, d.distance,
                                       d.angle, d.cbvar.get(), name = "Dirac")
        
    def generatehomogeneous(self, event = None):
        d = ActionGUI.HomogeneousDialog(self, title = "Homogeneous")
        if d.width: # not canceled
            ActionGUI.HomogeneousImageWindow(self, d.width, d.height, d.value,
                                             name = "Homogeneous")
        
    def generatesin(self, event = None):
        d = ActionGUI.SinDialog(self, title = "Sin / Cos")
        if d.width: # not canceled
            ActionGUI.SinImageWindow(self, d.width, d.height, d.frequencyx,
                                     d.phasex, d.frequencyy, d.phasey,
                                     name = "Sin / Cos")

    def generatesiemens(self, event = None):
        d = ActionGUI.SiemensDialog(self, title = "Siemens Star")
        if d.width: # not cancelled
            ActionGUI.SiemensImageWindow(self, d.width, d.edges,
                                         name = "Siemens Star " + str(d.edges))
        
    def about(self):
        "Help/About was chosen from the menu"
        ActionGUI.AboutDialog(self, title = 'About')

    def exit(self, event = None):
        self.quit()

if __name__ == "__main__":
    #
    # the main window
    #
    
    ImageWorkshop.workshop = Workshop()
    
    # start the main loop
    ImageWorkshop.workshop.mainloop()
    
